import 'dart:io';
import 'dart:convert';
import 'dart:async';
import 'package:dart_config/default_server.dart';

const String buildGradleFile = "android/app/build.gradle";

main(List<String> arguments) {
  Future<Map> conf = loadConfig("pubspec.yaml");
  conf.then((Map config) {
    if (config['flutter_version']['version_string'] != null) {
      //validate version
      if(isValidVersion(config['flutter_version']['version_string'])) {
        //update android build.gradle
        updateAndroid(config['flutter_version']['version_string']);
        //update ios Info.plist

      }
    } else {
      print("No version config found in pubspec.yaml");
    }
  });
}

bool isValidVersion(String s) {
  RegExp ex = new RegExp(r"^\d+.\d+.\d+$");
  return ex.hasMatch(s);
}

updateAndroid(String versionString) {

    final file = new File(buildGradleFile);
    Stream<List<int>> inputStream = file.openRead();
    StringBuffer fileOutput = new StringBuffer();

    inputStream.transform(UTF8.decoder).transform(new LineSplitter()).listen((String line) {
      //find version code
      if(line.trim().startsWith("versionCode")) {
        int versionCode = int.parse(line.trim().split(" ").elementAt(1));
        print("Found version code: " + versionCode.toString());
        //increment version code
        line = line.substring(0, line.length - versionCode.toString().length);
        versionCode++;
        line = line + versionCode.toString();
        print("Line with new version code: " + line);
        fileOutput.write(line);

      } else if(line.trim().startsWith("versionName")) {
        String oldVersion = line.trim().split(" ").elementAt(1);
        print("Found version name:" + oldVersion);
        line = line.substring(0, line.length - oldVersion.length);
        line = line + "\"" + versionString + "\"";
        print("Line with new version name: " + line);
        fileOutput.write(line);
      } else {
        fileOutput.write(line);
      }
    });

    print(fileOutput.toString());

    //new File(buildGradleFile).writeAsString(fileOutput);
  }